/* $Id: altq_ppp.c,v 1.1 1998/06/08 09:56:47 kjc Exp kjc $ */
/*
 * Copyright (C) 1997, 1998
 *	Sony Computer Science Laboratory Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY SONY CSL AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL SONY CSL OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include "command.h"
#include "mbuf.h"
#include "log.h"
#include "loadalias.h"
#include "defs.h"
#include "vars.h"
#include "filter.h"
#include "timer.h"
#include "ip.h"

#include "altq_ppp.h"

static struct altq _altq;
struct altq *altqp = &_altq;

int altq_debug = 0;

int altq_attach(enqueue, dequeue, account, type)
	int (*enqueue)(struct mbuf *);
	struct mbuf *(*dequeue)(void);
	int (*account)(struct mbuf *);
	int	type;
{
	if (altqp->altq_type != ALTQT_NONE)
		return (-1);

	IpFlushOutput();

	altqp->altq_enqueue = enqueue;
	altqp->altq_dequeue = dequeue;
	altqp->altq_account = account;
	altqp->altq_type = type;
	altqp->altq_flags = 0;

	return (0);
}

int altq_detach(type)
	int	type;
{
	if (altqp->altq_type != type)
		return (-1);
	altqp->altq_enqueue = NULL;
	altqp->altq_dequeue = NULL;
	altqp->altq_account = NULL;
	altqp->altq_type = ALTQT_NONE;
	altqp->altq_flags = 0;
	return (0);
}

int
altq_enable(type)
	int	type;
{
	if (altqp->altq_type != type)
		return (-1);

	altqp->altq_flags |= ALTQF_ENABLE;
	return (0);
}

int
altq_disable(type)
	int	type;
{
	if (altqp->altq_type != type)
		return (-1);

	altqp->altq_flags &= ~ALTQF_ENABLE;
	return (0);
}

