/*
 * Copyright (C) 1997, 1998
 *	Sony Computer Science Laboratory Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY SONY CSL AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL SONY CSL OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: altq_red.h,v 1.1 1998/06/08 09:56:47 kjc Exp kjc $
 */

#ifndef _ALTQ_RED_H_
#define _ALTQ_RED_H_

/* red flags */
#define REDF_ECN4	0x01	/* use packet marking for IPv4 packets */
#define REDF_ECN6	0x02	/* use packet marking for IPv6 packets */
#define REDF_ECN	(REDF_ECN4 | REDF_ECN6)

typedef struct red {
	int red_pkttime; 	/* average packet time in micro sec
				   used for idle calibration */
	int red_flags;		/* red flags */

	/* red parameters */
	int red_weight;		/* weight for EWMA */
	int red_wshift;		/* log(red_weight) */
	int red_avgshift;	/* log(red_weight) + fixed-point offset */
	int red_inv_pmax;	/* inverse of max drop probability */

	int red_avg;		/* queue length average scaled by avgshift */
	int red_count; 	  	/* packet count since the last dropped/marked
				   packet */
	int red_idle;		/* queue was empty */
	int red_old;		/* avg is above th_min */
	struct timeval red_last;  /* timestamp when the queue becomes idle */

	struct {
		quad_t xmit_packets;
		quad_t xmit_bytes;
		quad_t drop_packets;
		quad_t drop_bytes;
		quad_t drop_forced;
		quad_t drop_unforced;
		quad_t marked_packets;
	} red_stats;
} red_t;

typedef struct red_queue {
	struct red_queue *rq_next;	/* next red_state in the list */
	struct ifnet *rq_ifp;		/* backpointer to ifnet */

	class_queue_t *rq_q;

	red_t *rq_red;
} red_queue_t;

extern int red_init __P((red_t *, int, int));
extern int red_addq __P((red_t *, class_queue_t *, struct mbuf *));
extern struct mbuf *red_getq __P((red_t *, class_queue_t *));

#endif /* _ALTQ_RED_H_ */
